--[[--------------------------------------------------------------------
	OnScreenHealth
	Minimal text-based HUD for the player, pet, and target units.
	by Phanx < addons@phanx.net >
	Copyright  20072010 Phanx. Some rights reserved. See LICENSE.txt for details.
	http://www.wowinterface.com/downloads/info7470-OnScreenHealth.html
	http://wow.curse.com/downloads/wow-addons/details/onscreenhealth.aspx
----------------------------------------------------------------------]]

local ADDON_NAME, ns = ...

local OnScreenHealth = ns.OnScreenHealth
if not OnScreenHealth then return end

local frames = ns.unitFrames

local function getColor(info)
	local v = OnScreenHealth.db.profile[ info[#info] ]
	if not v then
		return
	elseif type(v) == "table" then
		return unpack(v)
	else
		return tonumber(v:sub(5, 6), 16) / 255, tonumber(v:sub(7, 8), 16) / 255, tonumber(v:sub(9, 10), 16) / 255
	end
end

local function setColor(info, r, g, b)
	local k = info[#info]
	local t = OnScreenHealth.db.profile
	if type(t[k]) == "table" then
		t[k][1] = r
		t[k][2] = g
		t[k][3] = b
	else
		t[k] = ("|cff%02x%02x%02x"):format(r * 255, g * 255, b * 255)
	end
	if k:match( "Health" ) then
		OnScreenHealth:UpdateAll("health")
	else
		-- Currently only using this for health and mana colors
		-- Will require changes if used for other color types!
		OnScreenHealth:UpdateAll("power")
	end
end

local function getPowerColor(info)
	local v = OnScreenHealth.db.profile.powerColors[ info[#info] ]
	if not v then return end
	return tonumber(v:sub(5, 6), 16) / 255, tonumber(v:sub(7, 8), 16) / 255, tonumber(v:sub(9, 10), 16) / 255
end

local function setPowerColor(info, r, g, b)
	local k = info[#info]
	local t = OnScreenHealth.db.profile.powerColors
	if not t[k] then return end
	t[k] = ("|cff%02x%02x%02x"):format(r * 255, g * 255, b * 255)
	OnScreenHealth:UpdateAll("power")
end

local function getResourceColor(info)
	local v = OnScreenHealth.db.profile.resourceColors[ info[#info] ]
	if not v then return end
	return tonumber(v:sub(5, 6), 16) / 255, tonumber(v:sub(7, 8), 16) / 255, tonumber(v:sub(9, 10), 16) / 255
end

local function setResourceColor(info, r, g, b)
	local k = info[#info]
	local t = OnScreenHealth.db.profile.resourceColors
	if not t[k] then return end
	t[k] = ("|cff%02x%02x%02x"):format(r * 255, g * 255, b * 255)
	OnScreenHealth:UpdateAll("resource")
end

local function getPos(info)
	return OnScreenHealth.db.profile[ info[#info] ]
end

local function setPos(info, v)
	OnScreenHealth.db.profile[ info[#info] ] = v
	OnScreenHealth:ApplyPositions()
end

local classToResource = {
	["DEATHKNIGHT"] = {["RUNES"] = true},
	["DRUID"] = {["COMBO_POINTS"] = true, ["ECLIPSE"] = true},
	["HUNTER"] = false,
	["MONK"] = {["CHI"] = true},
	["MAGE"] = false,
	["PALADIN"] = {["HOLY_POWER"] = true},
	["PRIEST"] = {["SHADOW_ORBS"] = true},
	["ROGUE"] = {["COMBO_POINTS"] = true},
	["SHAMAN"] = false,
	["WARRIOR"] = false,
	["WARLOCK"] = {["SOUL_SHARDS"] = true, ["BURNING_EMBERS"] = true, ["DEMONIC_FURY"] = true},
}
local function IsResourceClass(class, resource)
	if not resource then
		return classToResource[class]
	else
		return classToResource[class][resource]
	end
end

-- Hack to make values show up in order
local orderedStrata = {
	["1-BACKGROUND"] = "BACKGROUND",
	["2-LOW"] = "LOW",
	["3-MEDIUM"] = "MEDIUM",
	["4-HIGH"] = "HIGH",
}

function OnScreenHealth:LoadOptions(class)
	if self.optionsPanel then return end

	local L = ns.L
	if GetLocale() == "enUS" then
		L["Text Substitution Tokens"] = "\n$p = percent \n$c = current \n$m = maximum \n$d = missing \n$c:s = short current \n$m:s = short maximum \n$d:s = short missing"
	end
	local db = self.db.profile
	local Debug = ns.Debug
	local SharedMedia = LibStub("LibSharedMedia-3.0")

	local options = {
		type = "group", childGroups = "tab",
		args = {
			player = {
				order = 1,
				name = L["Player"],
				desc = L["Basic options for the player display."],
				type = "group",
				args = {
					enable = {
						order = 110,
						name = L["Enable"],
						desc = L["Enable the player display."],
						type = "toggle",
						get = function() return db.playerEnable end,
						set = function(info, v)
							db.playerEnable = v
							self:UpdatePlayer()
							self:UNIT_PET("player")
							ns.units["player"] = v
							ns.units["vehicle"] = v
						end
					},
					showDead = {
						order = 120,
						name = L["Show Dead"],
						desc = L["Show the player display while you are dead."],
						type = "toggle",
						get = function() return db.playerShowDead end,
						set = function(info, v)
							db.playerShowDead = v
							self:UpdateDeathEvents()
							self:UpdatePlayer()
							self:UNIT_PET("player")
						end
					},
					showOOC = {
						order = 130,
						name = L["Show OOC"],
						desc = L["Show the player display while you are out of combat."],
						type = "toggle",
						get = function() return db.playerShowOOC end,
						set = function(info, v)
							db.playerShowOOC = v
							self:UpdateCombatEvents()
							self:UpdatePlayer()
							self:UNIT_PET("player")
						end
					},
					showCombatStatus = {
						order = 140,
						name = L["Show Combat Status"],
						desc = L["Show an indicator above player health while you are in combat."],
						type = "toggle",
						get = function() return db.playerShowCombatStatus end,
						set = function(info, v)
							db.playerShowCombatStatus = v
							self:UpdateCombatEvents()
							self:UpdatePlayer()
						end,
					},
					frequentUpdates = {
						order = 150,
						name = L["Frequent Power and Resource Updates"],
						desc = L["Set the rate of updating the power and resource text."],
						type = "range", min = 0, max = 1.0, step = 0.1,
						get = function() return db.playerFrequentUpdates end,
						set = function(info, v)
							db.playerFrequentUpdates = v
						end
					}
				}
			},
			pet = {
				order = 2,
				name = L["Pet"],
				desc = L["Basic options for the pet display."],
				type = "group",
				args = {
					enable = {
						order = 210,
						name = L["Enable"],
						desc = L["Enable the pet display."],
						type = "toggle",
						get = function() return db.petEnable end,
						set = function(info, v)
							db.petEnable = v
							self:UNIT_PET("player")
							ns.units["pet"] = v
						end,
					},
					showDead = {
						order = 220,
						name = L["Show Dead"],
						desc = L["Show the pet display while your pet is dead."],
						type = "toggle",
						get = function() return db.petShowDead end,
						set = function(info, v)
							db.petShowDead = v
							self:UNIT_PET("player")
						end
					}
				}
			},
			target = {
				order = 3,
				name = L["Target"],
				desc = L["Basic options for the target display."],
				type = "group",
				args = {
					enable = {
						order = 310,
						name = L["Enable"],
						desc = L["Enable the target display."],
						type = "toggle",
						get = function() return db.targetEnable end,
						set = function(info, v)
							db.targetEnable = v
							self:PLAYER_TARGET_CHANGED()
							ns.units["target"] = v
						end
					},
					showDead = {
						order = 320,
						name = L["Show Dead"],
						desc = L["Show the target display while your target is dead or offline."],
						type = "toggle",
						get = function() return db.targetShowDead end,
						set = function(info, v)
							db.targetShowDead = v
							self:PLAYER_TARGET_CHANGED()
						end
					},
					showOOC = {
						order = 330,
						name = L["Show OOC"],
						desc = L["Show the target display while you are out of combat."],
						type = "toggle",
						get = function() return db.targetShowOOC end,
						set = function(info, v)
							db.targetShowOOC = v
							self:UpdateCombatEvents()
							self:PLAYER_TARGET_CHANGED()
						end
					},
					showToT = {
						order = 350,
						name = L["Target of Target"],
						desc = L["Show the name of your target's target."],
						type = "toggle",
						get = function() return db.targetShowToT end,
						set = function(info, v)
							db.targetShowToT = v
						end
					}
				}
			},
			appearance = {
				order = 4,
				name = L["Appearance"],
				desc = L["Options related to general appearance."],
				type = "group",
				args = {
					alpha = {
						name = L["Alpha"],
						desc = L["Options related to transparency."],
						type = "group", dialogInline = true,
						args = {
							mode = {
								order = 411,
								name = L["Mode"],
								desc = L["Choose how transparency should be set."],
								type = "select", values = { ["FIXED"] = L["Fixed Alpha"], ["COMBAT"] = L["Combat Fade"], ["HEALTH"] = L["Health Fade"] },
								get = function() return db.alphaMode end,
								set = function(info, v)
									db.alphaMode = v
									self:UpdateCombatEvents()
									if v == "COMBAT" then
										local a = UnitAffectingCombat("player") and db.alphaCombat or db.alphaOOC
										frames.player:SetAlpha(a)
										frames.target:SetAlpha(a)
										frames.pet:SetAlpha(a)
									else
										if v == "HEALTH" then
											self.UpdateHealth()
										else
											local a = db.alphaFixed
											frames.player:SetAlpha(a)
											frames.target:SetAlpha(a)
											frames.pet:SetAlpha(a)
										end
									end
								end,
							},
							fixed = {
								order = 412,
								name = L["Alpha"],
								desc = L["Set a fixed transparency for the displays."],
								hidden = function() return db.alphaMode ~= "FIXED" end,
								type = "range", min = 0.05, max = 1.00, step = 0.01, bigStep = 0.05, isPercent = true,
								get = function() return db.alphaFixed end,
								set = function(info, v)
									db.alphaFixed = v
									if db.alphaMode == "FIXED" then
										frames.player:SetAlpha(v)
										frames.target:SetAlpha(v)
										frames.pet:SetAlpha(v)
									end
								end,
							},
							combat = {
								order = 413,
								name = L["Alpha In Combat"],
								desc = L["Set the transparency level to use in combat."],
								hidden = function() return db.alphaMode ~= "COMBAT" end,
								type = "range", min = 0.05, max = 1.00, step = 0.01, bigStep = 0.05, isPercent = true,
								get = function() return db.alphaCombat end,
								set = function(info, v)
									db.alphaCombat = v
									if db.alphaMode == "COMBAT" and UnitAffectingCombat("player") then
										frames.player:SetAlpha(v)
										frames.target:SetAlpha(v)
										frames.pet:SetAlpha(v)
									end
								end,
							},
							ooc = {
								order = 414,
								name = L["Alpha Out Of Combat"],
								desc = L["Set the transparency level to use out of combat."],
								hidden = function() return db.alphaMode ~= "COMBAT" end,
								type = "range", min = 0.05, max = 1.00, step = 0.01, bigStep = 0.05, isPercent = true,
								get = function() return db.alphaOOC end,
								set = function(info, v)
									db.alphaOOC = v
									if db.alphaMode == "COMBAT" and not UnitAffectingCombat("player") then
										frames.player:SetAlpha(v)
										frames.target:SetAlpha(v)
										frames.pet:SetAlpha(v)
									end
								end,
							},
						}
					},
					colorMode = {
						name = L["Health Color Mode"],
						desc = L["Choose how to color the unit's health."],
						type = "select", values = { ["FIXED"] = L["Solid Color Thresholds"], ["GRADIENT"] = L["Smooth Color Gradient"], ["CLASS"] = L["Player Class"] },
						get = function() return db.colorHealthMode end,
						set = function(info, v)
							db.colorHealthMode = v
							self:UpdateAll("health")
						end
					},
					showPower = {
						name = L["Show Power Text"],
						desc = L["Show text for mana, rage, energy, runic power and focus."],
						type = "toggle",
						get = function() return db.showPower end,
						set = function(info, v)
							db.showPower = v
							if v then
								self.UpdatePower()
							else
								frames.player.Power:SetText()
								frames.target.Power:SetText()
								frames.pet.Power:SetText()
							end
						end
					},
					showResource = IsResourceClass(class) and {
						name = L["Show Resource Text"],
						desc = L["Show text for combo points, holy power, runes, shadow orbs, burning embers, eclipse and chi."], -- not needed for HUNTER, WARRIOR, MAGE, SHAMAN
						type = "toggle",
						get = function() return db.showResource end,
						set = function(info, v)
							db.showResource = v
							self:UpdateResourceEvents()
							if v then
								self.UpdateResource()
							else
								frames.player.Resource:SetText()
								frames.target.ComboPoints:SetText()
							end
						end
					} or nil,
					textAlign = {
						name = L["Text Alignment"],
						desc = L["Set the text alignment."],
						type = "select", values = { ["CENTER"] = L["Center"], ["INSIDE"] = L["Inside"], ["OUTSIDE"] = L["Outside"] },
						get = function() return db.textAlign end,
						set = function(info, v)
							db.textAlign = v
							self:ApplyPositions()
						end
					},
					textFormat = {
						name = L["Text Format"],
						desc = L["Set the format for displayed text."],
						type = "group", dialogInline = true,
						args = {
							health = {
								name = L["Health Format"],
								desc = L["Set the formats for health text."],
								type = "group", dialogInline = true,
								get = function(info) return db[ info[#info] ] end,
								set = function(info, v)
									db[ info[#info] ] = v
									self:UpdateAll("health")
								end,
								args = {
									textFormatHealthMonster = {
										name = L["NPC Health"],
										desc = L["Set the format for NPCs' health text."]..L["Text Substitution Tokens"],
										type = "input",
										usage = "Enter a text format for NPC health.",
									},
									textFormatHealthPlayer = {
										name = L["Player Health"],
										desc = L["Set the format for players' health text."]..L["Text Substitution Tokens"],
										type = "input",
										usage = "Enter a text format for player health.",
									}
								}
							},
							power = {
								name = L["Power Format"],
								desc = L["Set the formats for power text."],
								type = "group", dialogInline = true,
								get = function(info) return db[ info[#info] ] end,
								set = function(info, v)
									db[ info[#info] ] = v
									self:UpdateAll("power")
								end,
								args = {
									textFormatPowerMonster = {
										name = L["NPC Power"],
										desc = L["Set the format for NPCs' power text."]..L["Text Substitution Tokens"],
										type = "input",
										usage = "Enter a text format for NPC power.",
									},
									textFormatPowerPlayer = {
										name = L["Player Power"],
										desc = L["Set the format for players' power text."]..L["Text Substitution Tokens"],
										type = "input",
										usage = "Enter a text format for player power.",
									}
								}
							}
						}
					},
				}
			},
			font = {
				order = 5,
				name = L["Font"],
				desc = L["Options relating to the font used."],
				type = "group",
				args = {
					face = {
						order = 510,
						name = L["Typeface"],
						desc = L["Set the typeface to be used."],
						type = "select", values = AceGUIWidgetLSMlists.font, dialogControl = "LSM30_Font",
						get = function() return db.fontFace end,
						set = function(info, v)
							db.fontFace =  v
							self:ApplyFonts()
						end
					},
					outline = {
						order = 520,
						name = L["Outline"],
						desc = L["Set the text outline to be used."],
						type = "select", values = { ["NONE"] = L["None"], ["OUTLINE"] = L["Thin"], ["THICKOUTLINE"] = L["Thick"] },
						get = function() return db.fontOutline end,
						set = function(info, v)
							db.fontOutline = v
							self:ApplyFonts()
						end
					},
					shadow = {
						order = 530,
						name = L["Shadow"],
						desc = L["Enable a text shadow."],
						type = "toggle",
						get = function() return db.fontShadow end,
						set = function(info, v)
							db.fontShadow = v

							local s1, s2
							if v then
								s1, s2 = 2, -2
							else
								s1, s2 = 0, 0
							end

							frames.player.Health:SetShadowOffset(s1, s2)
							frames.player.Power:SetShadowOffset(s1, s2)
							frames.player.Combat:SetShadowOffset(s1, s2)
							frames.player.Resource:SetShadowOffset(s1, s2)

							frames.target.Health:SetShadowOffset(s1, s2)
							frames.target.Power:SetShadowOffset(s1, s2)
							frames.target.Target:SetShadowOffset(s1, s2)
							frames.target.ComboPoints:SetShadowOffset(s1, s2)

							frames.pet.Health:SetShadowOffset(s1, s2)
							frames.pet.Power:SetShadowOffset(s1, s2)
						end
					},
					size = {
						order = 540,
						name = L["Size"],
						desc = L["Options relating to font size."],
						type = "group", dialogInline = true,
						args = {
							health = {
								order = 541,
								name = L["Health"],
								desc = L["Set the font size used for health text."],
								type = "range", min = 8, max = 29, step = 1,
								get = function() return db.fontSizeHealth end,
								set = function(info, v)
									db.fontSizeHealth = v
									local face, outline = SharedMedia:Fetch("font", db.fontFace), db.fontOutline
									frames.player.Health:SetFont(face, v, outline)
									frames.target.Health:SetFont(face, v, outline)
									frames.pet.Health:SetFont(face, db.fontSizePet * v, outline)
								end
							},
							power = {
								order = 542,
								name = L["Power"],
								desc = L["Set the font size used for power text."],
								type = "range", min = 8, max = 29, step = 1,
								get = function() return db.fontSizePower end,
								set = function(info, v)
									db.fontSizePower = v
									local face, outline = SharedMedia:Fetch("font", db.fontFace), db.fontOutline
									frames.player.Power:SetFont(face, v, outline)
									frames.target.Power:SetFont(face, v, outline)
									frames.pet.Power:SetFont(face, db.fontSizePet * v, outline)
								end
							},
							resource = IsResourceClass(class) and {
								order = 543,
								name = L["Resource"],
								desc = L["Set the font size used for secondary resources."],
								type = "range", min = 8, max = 29, step = 1,
								get = function() return db.fontSizeResource end,
								set = function(info, v)
									db.fontSizeResource = v
									frames.player.Resource:SetFont(SharedMedia:Fetch("font", db.fontFace), v, db.fontOutline)
									frames.target.ComboPoints:SetFont(SharedMedia:Fetch("font", db.fontFace), v, db.fontOutline)
								end
							} or nil,
							target = {
								order = 544,
								name = L["Target Name"],
								desc = L["Set the font size used for target name text."],
								type = "range", min = 8, max = 29, step = 1,
								get = function() return db.fontSizeTarget end,
								set = function(info, v)
									db.fontSizeTarget = v
									frames.target.Target:SetFont(SharedMedia:Fetch("font", db.fontFace), v, db.fontOutline)
								end
							},
							pet = {
								order = 545,
								name = L["Pet Scale"],
								desc = L["Set the scale for pet health and power text compared to player text."],
								type = "range", min = 0.1, max = 1.0, step = 0.1, isPercent = true,
								get = function() return db.fontSizePet end,
								set = function(info, v)
									db.fontSizePet = v
									local face, outline = SharedMedia:Fetch("font", db.fontFace), db.fontOutline
									frames.pet.Health:SetFont(face, db.fontSizeHealth * v, outline)
									frames.pet.Power:SetFont(face, db.fontSizePower * v, outline)
								end
							}
						}
					}
				}
			},
			colors = {
				order = 6,
				name = L["Colors"],
				type = "group",
				args = {
					health = {
						name = L["Health"],
						type = "group", dialogInline = true,
						get = getColor,
						set = setColor,
						args = {
							colorHealthMax = {
								order = 611,
								name = L["Full Health"],
								desc = string.format(L["Set the color used for %s."], L["Full Health"]),
								type = "color",
							},
							colorHealthMid = {
								order = 612,
								name = L["Medium Health"],
								desc = string.format(L["Set the color used for %s."], L["Medium Health"]),
								type = "color",
							},
							colorHealthMin = {
								order = 613,
								name = L["Low Health"],
								desc = string.format(L["Set the color used for %s."], L["Low Health"]),
								type = "color",
							}
						}
					},
					power = {
						name = L["Power"],
						type = "group", dialogInline = true,
						get = getPowerColor,
						set = setPowerColor,
						args = {
							MANA = {
								order = 621,
								name = L["Mana"],
								desc = string.format(L["Set the color used for %s."], L["Mana"]),
								type = "color",
							},
							RAGE = {
								order = 622,
								name = L["Rage"],
								desc = string.format(L["Set the color used for %s."], L["Rage"]),
								type = "color",
							},
							FOCUS = {
								order = 623,
								name = L["Focus"],
								desc = string.format(L["Set the color used for %s."], L["Focus"]),
								type = "color",
							},
							ENERGY = {
								order = 624,
								name = L["Energy"],
								desc = string.format(L["Set the color used for %s."], L["Energy"]),
								type = "color",
							},
							RUNIC_POWER = {
								order = 625,
								name = L["Runic Power"],
								desc = string.format(L["Set the color used for %s."], L["Runic Power"]),
								type = "color",
							},
						}
					},
					resource = IsResourceClass(class) and {
						name = L["Resource"],
						type = "group", dialogInline = true,
						get = getResourceColor,
						set = setResourceColor,
						args = {
							COMBO_POINTS = IsResourceClass(class,"COMBO_POINTS") and {
								order = 631,
								name = L["Combo Points"],
								desc = string.format(L["Set the color used for %s."], L["Combo Points"]),
								type = "color",
							} or nil,
							HOLY_POWER = IsResourceClass(class,"HOLY_POWER") and {
								order = 632,
								name = L["Holy Power"],
								desc = string.format(L["Set the color used for %s."], L["Holy Power"]),
								type = "color",
							} or nil,
							CHI = IsResourceClass(class,"CHI") and {
								order = 633,
								name = L["Chi"],
								desc = string.format(L["Set the color used for %s."], L["Chi"]),
								type = "color",
							} or nil,
							SHADOW_ORBS = IsResourceClass(class,"SHADOW_ORBS") and {
								order = 634,
								name = L["Shadow Orbs"],
								desc = string.format(L["Set the color used for %s."], L["Shadow Orbs"]),
								type = "color",
							} or nil,
							SOUL_SHARDS = IsResourceClass(class,"SOUL_SHARDS") and {
								order = 635,
								name = L["Soul Shards"],
								desc = string.format(L["Set the color used for %s."], L["Soul Shards"]),
								type = "color",
							} or nil,
							BURNING_EMBERS = IsResourceClass(class,"BURNING_EMBERS") and {
								order = 636,
								name = L["Burning Embers"],
								desc = string.format(L["Set the color used for %s."], L["Burning Embers"]),
								type = "color",
							} or nil,
							DEMONIC_FURY = IsResourceClass(class,"DEMONIC_FURY") and {
								order = 637,
								name = L["Demonic Fury"],
								desc = string.format(L["Set the color used for %s."], L["Demonic Fury"]),
								type = "color",
							} or nil,
							ECLIPSE_SOLAR = IsResourceClass(class,"ECLIPSE") and {
								order = 638,
								name = L["Solar Energy"],
								desc = string.format(L["Set the color used for %s."], L["Solar Energy"]),
								type = "color",
							} or nil,
							ECLIPSE_LUNAR = IsResourceClass(class,"ECLIPSE") and {
								order = 639,
								name = L["Lunar Energy"],
								desc = string.format(L["Set the color used for %s."], L["Lunar Energy"]),
								type = "color",
							} or nil,
							RUNES1 = IsResourceClass(class,"RUNES") and {
								order = 640,
								name = L["Blood Rune"],
								desc = string.format(L["Set the color used for %s."], L["Blood Rune"]),
								type = "color",
							} or nil,
							RUNES2 = IsResourceClass(class,"RUNES") and {
								order = 641,
								name = L["Unholy Rune"],
								desc = string.format(L["Set the color used for %s."], L["Unholy Rune"]),
								type = "color",
							} or nil,
							RUNES3 = IsResourceClass(class,"RUNES") and {
								order = 642,
								name = L["Frost Rune"],
								desc = string.format(L["Set the color used for %s."], L["Frost Rune"]),
								type = "color",
							} or nil,
							RUNES4 = IsResourceClass(class,"RUNES") and {
								order = 643,
								name = L["Death Rune"],
								desc = string.format(L["Set the color used for %s."], L["Death Rune"]),
								type = "color",
							} or nil,
						}
					} or nil,
					absent = {
						name = L["Absent"],
						desc = string.format(L["Set the color used for %s."], L["dead and offline units"]),
						type = "color",
						get = function()
							local v = db.colorAbsent
							return tonumber(v:sub(5, 6), 16) / 255, tonumber(v:sub(7, 8), 16) / 255, tonumber(v:sub(9, 10), 16) / 255
						end,
						set = function(info, r, g, b)
							db.colorAbsent = string.format("|cff%02x%02x%02x", r*255, g*255, b*255)
							self.UpdateHealth()
						end
					},
					combatStatus = {
						name = L["Combat Status"],
						desc = string.format(L["Set the color used for %s."], L["the player combat status indicator."]),
						type = "color",
						get = function()
							local tmp1 = db.colorCombatStatus
							return tonumber(tmp1:sub(5, 6), 16) / 255, tonumber(tmp1:sub(7, 8), 16) / 255, tonumber(tmp1:sub(9, 10), 16) / 255
						end,
						set = function(info, r, g, b)
							db.colorCombatStatus = string.format("|cff%02x%02x%02x", r*255, g*255, b*255)
							self:UpdateCombatStatus("player")
						end
					},
					targetSelf = {
						name = L["Target Self"],
						desc = string.format(L["Set the color used for %s."], L["your name when you are your target's target"]),
						type = "color",
						get = function()
							local tmp1 = db.colorTargetSelf
							return tonumber(tmp1:sub(5, 6), 16) / 255, tonumber(tmp1:sub(7, 8), 16) / 255, tonumber(tmp1:sub(9, 10), 16) / 255
						end,
						set = function(info, r, g, b)
							db.colorTargetSelf = string.format("|cff%02x%02x%02x", r*255, g*255, b*255)
							self.UpdateToT()
						end
					}
				}
			},
			position = {
				order = 7,
				name = L["Position"],
				desc = L["Set the position of the displays on-screen."],
				type = "group",
				get = getPos,
				set = setPos,
				args = {
					posX = {
						order = 710,
						name = L["Horizontal"],
						desc = L["Set the horizontal distance from the center of the screen."],
						type = "range", min = 0, max = 800, step = 10, bigStep = 50,
					},
					posY = {
						order = 720,
						name = L["Vertical"],
						desc = L["Set the vertical distance from the center of the screen."],
						type = "range", min = -500, max = 500, step = 10, bigStep = 25,
					},
					posPet = {
						order = 730,
						name = L["Pet Offset"],
						desc = L["Set the distance to position the pet frame from the player frame."],
						type = "range", min = -200, max = 200, step = 10, bigStep = 25,
					},
					frameStrata = {
						order = 780,
						name = L["Frame Strata"],
						desc = L["Change which strata the text appears on. Useful if you wish to place the text over other frames."],
						type = "select", values = orderedStrata,
						get = function(info)
							for k, v in pairs(orderedStrata) do
								if v == db.frameStrata then
									return k
								end
							end
						end,
						set = function(info, v)
							for k, strata in pairs(orderedStrata) do
								if k == v then
									db.frameStrata = strata
									break
								end
							end
							for _, frame in pairs(frames) do
								frame:SetFrameStrata(db.frameStrata)
							end
						end,
					},
				}
			}
		}
	}

	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	options.args.profile.order = 8

	self.options = LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("OnScreenHealth", options)

	self.optionsPanel = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("OnScreenHealth")
	self.aboutPanel = LibStub("LibAboutPanel").new("OnScreenHealth", "OnScreenHealth")

	LibStub("AceConsole-3.0"):RegisterChatCommand("osh", function()
		InterfaceOptionsFrame_OpenToCategory(OnScreenHealth.aboutPanel)
		InterfaceOptionsFrame_OpenToCategory(OnScreenHealth.optionsPanel)
		InterfaceOptionsFrame:Raise()
	end)
end